@@ -1,11 +1,10 @@
 /*
 ################################################################################
 #
-# r8168 is the Linux device driver released for RealTek RTL8168B/8111B,
-# RTL8168C/8111C, RTL8168CP/8111CP, RTL8168D/8111D, and RTL8168DP/8111DP, and
-# RTK8168E/8111E Gigabit Ethernet controllers with PCI-Express interface.
+# r8168 is the Linux device driver released for Realtek Gigabit Ethernet
+# controllers with PCI-Express interface.
 #
-# Copyright(c) 2013 Realtek Semiconductor Corp. All rights reserved.
+# Copyright(c) 2014 Realtek Semiconductor Corp. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify it
 # under the terms of the GNU General Public License as published by the Free
@@ -32,6 +31,13 @@
  *  US6,570,884, US6,115,776, and US6,327,625.
  ***********************************************************************************/
 
+#include "r8168_dash.h"
+
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(3,10,0)
+#define NETIF_F_HW_VLAN_RX	NETIF_F_HW_VLAN_CTAG_RX
+#define NETIF_F_HW_VLAN_TX	NETIF_F_HW_VLAN_CTAG_TX
+#endif
+
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(3,8,0)
 #define __devinit
 #define __devexit
@@ -98,6 +104,25 @@
 #define RTL_NET_DEVICE_OPS(ops) dev->netdev_ops=&ops
 #endif
 
+#ifndef FALSE
+#define FALSE 0
+#endif
+
+#ifndef TRUE
+#define TRUE  1
+#endif
+
+#ifndef false
+#define false 0
+#endif
+
+#ifndef true
+#define true  1
+#endif
+
+//Hardware will continue interrupt 10 times after interrupt finished.
+#define RTK_KEEP_INTERRUPT_COUNT (10)
+
 //Due to the hardware design of RTL8111B, the low 32 bit address of receive
 //buffer must be 8-byte alignment.
 #ifndef NET_IP_ALIGN
@@ -111,7 +136,7 @@
 #define NAPI_SUFFIX ""
 #endif
 
-#define RTL8168_VERSION "8.036.00" NAPI_SUFFIX
+#define RTL8168_VERSION "8.038.00" NAPI_SUFFIX
 #define MODULENAME "r8168"
 #define PFX MODULENAME ": "
 
@@ -834,6 +859,10 @@ enum RTL8168_registers {
     FuncEvent       = 0xF0,
     FuncEventMask       = 0xF4,
     FuncPresetState     = 0xF8,
+    IBCR0           = 0xF8,
+    IBCR2           = 0xF9,
+    IBIMR0          = 0xFA,
+    IBISR0          = 0xFB,
     FuncForceEvent      = 0xFC,
 };
 
@@ -900,16 +929,18 @@ enum RTL8168_register_content {
     TxDMAShift = 8, /* DMA burst value (0-7) is shift this many bits */
     TxMACLoopBack = (1 << 17),  /* MAC loopback */
 
-    /* Config1 register p.24 */
-    PMSTS_En    = (1 << 13),
+    /* Config1 register */
     LEDS1       = (1 << 7),
     LEDS0       = (1 << 6),
     Speed_down  = (1 << 4),
     MEMMAP      = (1 << 3),
     IOMAP       = (1 << 2),
-    VPD     = (1 << 1),
+    VPD         = (1 << 1),
     PMEnable    = (1 << 0), /* Power Management Enable */
 
+    /* Config2 register */
+    PMSTS_En    = (1 << 5),
+
     /* Config3 register */
     Isolate_en  = (1 << 12), /* Isolate enable */
     MagicPacket = (1 << 5), /* Wake up when receives a Magic Packet */
@@ -1048,8 +1079,9 @@ enum _DescStatusBit {
     /* Tx private */
     /*------ offset 0 of tx descriptor ------*/
     LargeSend   = (1 << 27), /* TCP Large Send Offload (TSO) */
+    LargeSend_DP = (1 << 16), /* TCP Large Send Offload (TSO) */
     MSSShift    = 16,        /* MSS value position */
-    MSSMask     = 0x7ffU,    /* MSS value + LargeSend bit: 12 bits */
+    MSSMask     = 0x7FFU,    /* MSS value 11 bits */
     TxIPCS      = (1 << 18), /* Calculate IP checksum */
     TxUDPCS     = (1 << 17), /* Calculate UDP/IP checksum */
     TxTCPCS     = (1 << 16), /* Calculate TCP/IP checksum */
@@ -1173,8 +1205,8 @@ struct pci_resource {
     u16 resv_0x24_l;
     u16 resv_0x2c_h;
     u16 resv_0x2c_l;
-    u32 pci_nvidia_geforce_6200;
-    u32 pci_nvidia_geforce__6250_1;
+    u32 pci_sn_l;
+    u32 pci_sn_h;
 };
 
 struct rtl8168_private {
@@ -1216,13 +1248,18 @@ struct rtl8168_private {
     unsigned int rtl8168_rx_config;
     u16 cp_cmd;
     u16 intr_mask;
+    u16 timer_intr_mask;
     int phy_auto_nego_reg;
     int phy_1000_ctrl_reg;
     u8 org_mac_addr[NODE_ADDRESS_SIZE];
+    struct rtl8168_counters *tally_vaddr;
+    dma_addr_t tally_paddr;
+
 #ifdef CONFIG_R8168_VLAN
     struct vlan_group *vlgrp;
 #endif
     u8  wol_enabled;
+    u32 wol_opts;
     u8  efuse;
     u8  eeprom_type;
     u8  autoneg;
@@ -1230,6 +1267,7 @@ struct rtl8168_private {
     u16 speed;
     u16 eeprom_len;
     u16 cur_page;
+    u32 bios_setting;
 
     int (*set_speed)(struct net_device *, u8 autoneg, u16 speed, u8 duplex);
     void (*get_settings)(struct net_device *, struct ethtool_cmd *);
@@ -1242,6 +1280,92 @@ struct rtl8168_private {
     struct delayed_work task;
 #endif
     unsigned features;
+
+    u8 org_pci_offset_99;
+    u8 org_pci_offset_180;
+    u8 issue_offset_99_event;
+
+    u8 org_pci_offset_80;
+    u8 org_pci_offset_81;
+    u8 use_timer_interrrupt;
+
+    u8 in_open_fun;
+
+    u32 keep_intr_cnt;
+
+    u8  HwIcVerUnknown;
+    u8  NotWrRamCodeToMicroP;
+    u8  NotWrMcuPatchCode;
+    u8  HwHasWrRamCodeToMicroP;
+
+    u8 rtk_enable_diag;
+
+    //Dash+++++++++++++++++
+    u8 HwSuppDashVer;
+    u8 DASH;
+
+#ifdef ENABLE_DASH_SUPPORT
+    u16 AfterRecvFromFwBufLen;
+    u8 AfterRecvFromFwBuf[RECV_FROM_FW_BUF_SIZE];
+    u16 AfterSendToFwBufLen;
+    u8 AfterSendToFwBuf[SEND_TO_FW_BUF_SIZE];
+    u16 SendToFwBufferLen;
+    u32 SizeOfSendToFwBuffer ;
+    u32 SizeOfSendToFwBufferMemAlloc ;
+    u32 NumOfSendToFwBuffer ;
+
+    u8 OobReq;
+    u8 OobAck;
+    u32 OobReqComplete;
+    u32 OobAckComplete;
+
+    u8 RcvFwReqSysOkEvt;
+    u8 RcvFwDashOkEvt;
+    u8 SendFwHostOkEvt;
+
+    u8 DashFwDisableRx;
+
+    void *UnalignedSendToFwBufferVa;
+    void *SendToFwBuffer ;
+    u64 SendToFwBufferPhy ;
+    u8 SendingToFw;
+    u64 UnalignedSendToFwBufferPa;
+    PTX_DASH_SEND_FW_DESC TxDashSendFwDesc;
+    u64 TxDashSendFwDescPhy;
+    u8 *UnalignedTxDashSendFwDescVa;
+    u32 SizeOfTxDashSendFwDescMemAlloc;
+    u32 SizeOfTxDashSendFwDesc ;
+    u32 NumTxDashSendFwDesc ;
+    u32 CurrNumTxDashSendFwDesc ;
+    u64 UnalignedTxDashSendFwDescPa;
+
+    u32 NumRecvFromFwBuffer ;
+    u32 SizeOfRecvFromFwBuffer ;
+    u32 SizeOfRecvFromFwBufferMemAlloc ;
+    void *RecvFromFwBuffer ;
+    u64 RecvFromFwBufferPhy ;
+
+    void *UnalignedRecvFromFwBufferVa;
+    u64 UnalignedRecvFromFwBufferPa;
+    PRX_DASH_FROM_FW_DESC RxDashRecvFwDesc;
+    u64 RxDashRecvFwDescPhy;
+    u8 *UnalignedRxDashRecvFwDescVa;
+    u32 SizeOfRxDashRecvFwDescMemAlloc;
+    u32 SizeOfRxDashRecvFwDesc ;
+    u32 NumRxDashRecvFwDesc ;
+    u32 CurrNumRxDashRecvFwDesc ;
+    u64 UnalignedRxDashRecvFwDescPa;
+    u8 DashReqRegValue;
+    u16 HostReqValue;
+
+    u32 CmacResetIsrCounter;
+    u8 CmacResetIsr1st ;
+    u8 CmacResetIsr2nd ;
+    u8 CmacResetting ;
+    u8 CmacOobIssueCmacReset ;
+#endif
+
+    //Dash-----------------
 };
 
 enum eetype {
@@ -1279,6 +1403,7 @@ enum mcfg {
     CFG_METHOD_25,
     CFG_METHOD_26,
     CFG_METHOD_27,
+    CFG_METHOD_28,
     CFG_METHOD_MAX,
     CFG_METHOD_DEFAULT = 0xFF
 };
@@ -1292,14 +1417,16 @@ enum mcfg {
 //Ram Code Version
 #define NIC_RAMCODE_VERSION_CFG_METHOD_14 (0x0057)
 #define NIC_RAMCODE_VERSION_CFG_METHOD_16 (0x0055)
-#define NIC_RAMCODE_VERSION_CFG_METHOD_18 (0x0037)
-#define NIC_RAMCODE_VERSION_CFG_METHOD_20 (0x0035)
+#define NIC_RAMCODE_VERSION_CFG_METHOD_18 (0x0044)
+#define NIC_RAMCODE_VERSION_CFG_METHOD_20 (0x0044)
 #define NIC_RAMCODE_VERSION_CFG_METHOD_21 (0x0019)
 #define NIC_RAMCODE_VERSION_CFG_METHOD_24 (0x0001)
 #define NIC_RAMCODE_VERSION_CFG_METHOD_23 (0x0015)
 #define NIC_RAMCODE_VERSION_CFG_METHOD_26 (0x0012)
+#define NIC_RAMCODE_VERSION_CFG_METHOD_28 (0x0010)
 
 void mdio_write(struct rtl8168_private *tp, u32 RegAddr, u32 value);
+void mdio_prot_write(struct rtl8168_private *tp, u32 RegAddr, u32 value);
 void rtl8168_ephy_write(void __iomem *ioaddr, int RegAddr, int value);
 void OCP_write(struct rtl8168_private *tp, u8 mask, u16 Reg, u32 data);
 void OOB_notify(struct rtl8168_private *tp, u8 cmd);
@@ -1311,6 +1438,10 @@ u32 OCP_read(struct rtl8168_private *tp, u8 mask, u16 Reg);
 u32 rtl8168_eri_read(void __iomem *ioaddr, int addr, int len, int type);
 u16 rtl8168_ephy_read(void __iomem *ioaddr, int RegAddr);
 void OOB_mutex_unlock(struct rtl8168_private *tp);
+void Dash2DisableTx(struct rtl8168_private *tp);
+void Dash2EnableTx(struct rtl8168_private *tp);
+void Dash2DisableRx(struct rtl8168_private *tp);
+void Dash2EnableRx(struct rtl8168_private *tp);
 
 
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,34)
